from decimal import Decimal

from django.db import models


class Receipt(models.Model):
    created = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return f'Receipt(id={self.id})'

    def total(self) -> Decimal:
        return sum(item.cost for item in self.item_set.all())


class Item(models.Model):
    created = models.DateTimeField(auto_now_add=True)

    description = models.TextField()
    cost = models.DecimalField(max_digits=7, decimal_places=2)
    receipt = models.ForeignKey(Receipt, on_delete=models.CASCADE)

    def __str__(self):
        return f'Item(id={self.id}, description={self.description}, ' \
            f'cost={self.cost})'



from django.http import JsonResponse

from receipts.models import Receipt

def receipt_json(request):
    results = {
        'receipts':[],
    }

    for receipt in Receipt.objects.all():
        line = [str(receipt), []]
        for item in receipt.item_set.all():
            line[1].append(str(item))

        results['receipts'].append(line)

    return JsonResponse(results)



import os

readme = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'README.rst')
long_description = open(readme).read()

SETUP_ARGS = dict(
    name='realpython-django-receipts',
    version='1.0.2',
    description='Sample installable django app',
    long_description=long_description,
    url='https://github.com/realpython/django-receipts',
    license='MIT',
    include_package_data=True,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    python_requires=">=3.7",
    install_requires=[
        'Django>=3.0',
    ],
    test_suite='load_tests.get_suite',
)

if __name__ == '__main__':
    from setuptools import setup, find_packages

    SETUP_ARGS['packages'] = find_packages()
    setup(**SETUP_ARGS)



from decimal import Decimal 

from django.test import TestCase

from receipts.models import Receipt

class ReceiptTest(TestCase):
    fixtures = ['receipts.json', ]

    def test_receipt(self):
        receipt = Receipt.objects.get(id=1)
        total = receipt.total()

        expected = Decimal('37.55')
        self.assertEqual(expected, total)



# Generated by Django 3.0.4 on 2020-03-24 15:17

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Receipt',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True)),
            ],
        ),
        migrations.CreateModel(
            name='Item',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('description', models.TextField()),
                ('cost', models.DecimalField(decimal_places=2, max_digits=7)),
                ('receipt', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='receipts.Receipt')),
            ],
        ),
    ]



from django.contrib import admin

from receipts.models import Receipt, Item

@admin.register(Receipt)
class ReceiptAdmin(admin.ModelAdmin):
    pass


@admin.register(Item)
class ItemAdmin(admin.ModelAdmin):
    pass
